// --------------------------------------------//
// Author: Viktória Fördős <f-viktoria@elte.hu>//
// --------------------------------------------//

// To compile:
// -under OSX: gcc -Wall -fPIC -O3 -undefined dynamic_lookup -dynamiclib nif_demo.pp -I /usr/local/lib/erlang/usr/include -o ./priv/nif_demo
// -under UNIX: gcc -Wall -fPIC -O3 -shared nif_demo.cpp -I /usr/local/lib/erlang/usr/include -o ./priv/nif_demo

#include "erl_nif.h"

static ERL_NIF_TERM add_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]){
    // unpacking the given parameters
    int x,y,ret;
    if (!enif_get_int(env, argv[0], &x)) {
	    return enif_make_badarg(env);
    }
    if (!enif_get_int(env, argv[1], &y)) {
	    return enif_make_badarg(env);
    }
    // computation
    ret = x+y;
    // packing and returning the result
    return enif_make_int(env, ret);
}

static ERL_NIF_TERM mul_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]){
    // unpacking the given parameters
    int x,y,ret;
    if (!enif_get_int(env, argv[0], &x)) {
	    return enif_make_badarg(env);
    }
    if (!enif_get_int(env, argv[1], &y)) {
	    return enif_make_badarg(env);
    }
    // computation
    ret = x*y;
    // packing and returning the result
    return enif_make_int(env, ret);
}

static ErlNifFunc nif_funcs[] = {
    {"add", 2, add_nif},
    {"mul", 2, mul_nif}
};

ERL_NIF_INIT(nif_demo, nif_funcs, NULL, NULL, NULL, NULL)


